package com.mixel.docusphere.service;

import com.mixel.docusphere.dto.UserDTO;
import com.mixel.docusphere.entity.User;
import com.mixel.docusphere.repository.UserRepository;
import org.springframework.security.crypto.argon2.Argon2PasswordEncoder;
import org.springframework.security.crypto.keygen.KeyGenerators;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.Optional;
import java.util.UUID;

@Service
public class UserService {

    private final UserRepository userRepository;

    private final Argon2PasswordEncoder passwordEncoder;

    // Constructor
    public UserService(UserRepository userRepository) {
        this.passwordEncoder = new Argon2PasswordEncoder(16, 32, 1, 4096, 3);
        this.userRepository = userRepository;
    }

    public List<User> findAll() {
        return userRepository.findAll();
    }

    public Optional<User> findById(UUID id) {
        return userRepository.findById(id);
    }

    public User save(UserDTO userDTO) {
        User user = new User();
        user.setUsername(userDTO.getUsername());
        user.setName(userDTO.getName());
        user.setEmail(userDTO.getEmail());

        isPasswordAlreadySet(userDTO, user);
        return userRepository.save(user);
    }

    public Optional<User> update(UUID id, UserDTO userDTO){
        Optional<User> userOptional = userRepository.findById(id);
        if (userOptional.isPresent()) {
            User user = userOptional.get();
            user.setUsername(userDTO.getUsername());
            user.setName(userDTO.getName());
            user.setEmail(userDTO.getEmail());

            isPasswordAlreadySet(userDTO, user);
            return Optional.of(userRepository.save(user));
        }
        return Optional.empty();
    }

    private void isPasswordAlreadySet(UserDTO userDTO, User user) {
        if (userDTO.getPassword() != null && !userDTO.getPassword().isEmpty()) {
            final String salt = KeyGenerators.string().generateKey();
            user.setPasswordSalt(salt);
            final String saltedPassword = salt + userDTO.getPassword();
            user.setPasswordHash(passwordEncoder.encode(saltedPassword));
        }
    }

    public void deleteById(UUID id) {
        userRepository.deleteById(id);
    }
}